<div class="tree-file">
    {% if tree.report.success %}
        <a href="{{ setup['href_prefix'] }}{{ tree.report.target }}">{{ tree.report.target|basename }}</a>
        {% if tree.report.target != tree.from_source.as_posix() %}
            ({{ render_download(tree) }})
        {% endif %}
    {% else %}
        {{ render_download(tree) }}
        <span class="error">[Error : {{ tree.report.error }}]</span>
    {% endif %}

    ; last modified on {{ tree.last_modified().strftime('%c') }}.
    {{ render_readme(tree) }}
</div>
