<div class="tree-directory">
    <i>{{ tree.path.name }}</i> {{ render_readme(tree) }}
    <ul>
        {% for dir in tree.values()|sort %}
            <li>
                {{ render(dir, "tree.html") }}
            </li>
        {% endfor %}
    </ul>
</div>
